@@ -16,8 +16,8 @@
  * 
  * You should have received a copy of the GNU Library General Public
  * License along with the Mate Library; see the file COPYING.LIB.  If not,
- * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  * 
  * Author: Soren Sandmann <sandmann@redhat.com>
  */
@@ -486,8 +486,12 @@ fill_out_screen_info (Display *xdisplay,
     if (info->screen->rr_major_version == 1 && info->screen->rr_minor_version >= 3) {
         gdk_error_trap_push ();
         info->primary = XRRGetOutputPrimary (xdisplay, xroot);
+      #if GTK_CHECK_VERSION (3, 0, 0)
+	gdk_error_trap_pop_ignored ();
+      #else
 	gdk_flush ();
 	gdk_error_trap_pop (); /* ignore error */
+      #endif
     }
 #endif
 
@@ -746,8 +750,12 @@ mate_rr_screen_set_size (MateRRScreen *screen,
     gdk_error_trap_push ();
     XRRSetScreenSize (screen->xdisplay, screen->xroot,
 		      width, height, mm_width, mm_height);
+  #if GTK_CHECK_VERSION (3, 0, 0)
+    gdk_error_trap_pop_ignored ();
+  #else
     gdk_flush ();
     gdk_error_trap_pop (); /* ignore error */
+  #endif
 #endif
 }
 
